#ALL CREDIT GOES TO: https://github.com/LaraChicharo/flappy-bird
  
#!/usr/bin/env python

# This file is part of flascii_bird.
# Copyright (C) 2014 Cesar Saez

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 3.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import select
import sys
import termios
import time
import tty
from math import fmod, sqrt
from random import randint


class Vector(object):

    def __init__(self, x, y):
        super(Vector, self).__init__()
        self.x, self.y = x, y

    def __add__(self, vector):
        if isinstance(vector, self.__class__):
            return self.__class__(self.x + vector.x, self.y + vector.y)
        return super(Vector, self).__add__(vector)

    def __mul__(self, vector):
        if isinstance(vector, self.__class__):
            return self.__class__(self.x * vector.x, self.y * vector.y)
        return self.__class__(self.x * vector, self.y * vector)

    def __repr__(self):
        return "{0}, {1}".format(self.x, self.y)

    @property
    def length(self):
        return sqrt(self.x ** 2 + self.y ** 2)

    def normalize(self):
        _length = self.length
        self.x = self.x / _length
        self.y = self.y / _length


class Sprite(object):

    def __init__(self, shape):
        self.shape = shape
        self.pos = Vector(0, 0)
        self.vel = Vector(0, 0)

    @property
    def bbox(self):
        if not hasattr(self, "_bbox"):
            sp = self.shape.split("\n")
            self._bbox = Vector(max([len(x) for x in sp]), len(sp))
        return self._bbox

    @property
    def shape(self):
        if type(self._shape) in (list, tuple):
            index = 0 if self.vel.y > 0 else 1
            return self._shape[index]
        return self._shape

    @shape.setter
    def shape(self, value):
        self._shape = value

    def simulate(self, forces=None, max_speed=2):
        forces = forces or list()
        for f in forces:
            self.vel = self.vel + f  # time, mass == 1
        if self.vel.length > max_speed:
            self.vel.normalize()
            self.vel = self.vel * max_speed
        self.pos = self.pos + self.vel

    def collide(self, obstacles=None):
        obstacles = obstacles or list()
        for o in obstacles:
            condx = all([self.pos.x + self.bbox.x > o.pos.x,
                         self.pos.x < o.pos.x + o.bbox.x])
            condy = all([self.pos.y + self.bbox.y > o.pos.y,
                         self.pos.y < o.pos.y + o.bbox.y])
            if condx and condy:
                return True
        return False

    def draw(self, BG):
        BG = BG.split("\n")
        self.pos.x, self.pos.y = int(self.pos.x), int(self.pos.y)  # rasterize
        for i, s in enumerate(self.shape.split("\n")):
            x = BG[i + self.pos.y]
            if self.pos.x >= 0:
                x = x[:self.pos.x] + s + x[self.pos.x + len(s):]
            else:
                x = s[abs(self.pos.x):] + x[self.pos.x + len(s):] + (" " * 79)
            BG[i + self.pos.y] = x
        return "\n".join([ch[:79] for ch in BG][:25])


def is_data():
    return select.select([sys.stdin], [], [], 0) == ([sys.stdin], [], [])


def main():
    global KEY_PRESSED
    SCORE = 0
    STEP = 0.1
    GRAVITY = Vector(0, 1)
    KEY_STRENGHT = Vector(0, -2)
    # TERMINAL_SIZE = Vector(88899, 25)
    TERMINAL_SIZE = Vector(79, 25)

    BG = (" " * TERMINAL_SIZE.x + "\n") * TERMINAL_SIZE.y
    TUBES = list()
    GROUND = ("=" * TERMINAL_SIZE.x + "\n") + \
        ("." * TERMINAL_SIZE.x + "\n") * 3
    GROUND = Sprite(GROUND)
    GROUND.pos = Vector(0, 21)
    BIRD = Sprite(("== (.\n \___\\", " / (./\n===_/"))
    BIRD.pos = Vector(10, 0)

    t = 0

    while True:
        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(sys.stdin)
        tty.setcbreak(sys.stdin.fileno())

        t += 1
        time.sleep(STEP)
        if is_data():
            ch = sys.stdin.read(1)
            if ch == " ":
                KEY_PRESSED = True
        else:
            print("false")

        # if this setence in the game over, graph not right
        termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)

        # tubes
        if fmod(t, 51) == 0 or t == 1:
            pipe_height = randint(1, 7)
            up = Sprite("|      |\n" * pipe_height + "--------\n--------")
            up.pos = Vector(TERMINAL_SIZE.x, 0)
            TUBES.append(up)
            dn = Sprite("--------\n" * 2 + "|      |\n" * (8 - pipe_height))
            dn.shape = dn.shape[:-1]  # remove last \n
            dn.pos = Vector(TERMINAL_SIZE.x,
                            TERMINAL_SIZE.y + pipe_height - 8 - 6)
            TUBES.append(dn)
            TUBES = [x for x in TUBES if x.pos.x > -7]  # cleanup
        f = Vector(-1, 0)
        for x in TUBES:
            x.simulate([f], max_speed=1)

        # bird
        BIRD.simulate([GRAVITY])
        if KEY_PRESSED:
            KEY_PRESSED = False
            BIRD.vel = KEY_STRENGHT
            BIRD.simulate(max_speed=25)  # dont clamp velocity

        # score
        for x in TUBES:
            if x.pos.x == BIRD.pos.x - 9:
                SCORE += 0.5  # each tube
        SCORE = int(SCORE)

        # draw
        frame = GROUND.draw(BG)
        for x in TUBES:
            frame = x.draw(frame)
        s = Sprite(" SCORE: " + str(SCORE) + " ")
        s.pos = Vector(60, 23)
        docs = Sprite(" PRESS <SPACEBAR> ")
        docs.pos = Vector(5, 23)
        for x in (s, docs):
            frame = x.draw(frame)
        print(BIRD.draw(frame))

        # collisions
        colliders = list(TUBES)
        colliders.append(GROUND)
        if BIRD.collide(colliders) or BIRD.pos.y < 0:
            # game over
            print((" " * TERMINAL_SIZE.x + "\n") * TERMINAL_SIZE.y)
            print("GAME OVER")
            print("SCORE:", SCORE)
            return

# if __name__ == "__main__":
KEY_PRESSED = False
main()
assert False,'Game Over. Re-import flappy to play again.'



# Stubs for warnings

from typing import Any, Dict, List, NamedTuple, Optional, TextIO, Tuple, Type, Union
from types import ModuleType, TracebackType

def warn(message: Union[str, Warning], category: Optional[Type[Warning]] = ...,
         stacklevel: int = ...) -> None: ...
def warn_explicit(message: Union[str, Warning], category: Type[Warning],
                  filename: str, lineno: int, module: Optional[str] = ...,
                  registry: Optional[Dict[Union[str, Tuple[str, Type[Warning], int]], int]] = ...,
                  module_globals: Optional[Dict[str, Any]] = ...) -> None: ...
def showwarning(message: str, category: Type[Warning], filename: str,
                lineno: int, file: Optional[TextIO] = ...,
                line: Optional[str] = ...) -> None: ...
def formatwarning(message: str, category: Type[Warning], filename: str,
                  lineno: int, line: Optional[str] = ...) -> str: ...
def filterwarnings(action: str, message: str = ...,
                   category: Type[Warning] = ..., module: str = ...,
                   lineno: int = ..., append: bool = ...) -> None: ...
def simplefilter(action: str, category: Type[Warning] = ..., lineno: int = ...,
                 append: bool = ...) -> None: ...
def resetwarnings() -> None: ...

_Record = NamedTuple('_Record',
                     [('message', str),
                      ('category', Type[Warning]),
                      ('filename', str),
                      ('lineno', int),
                      ('file', Optional[TextIO]),
                      ('line', Optional[str])])

class catch_warnings:
    def __init__(self, *, record: bool = ...,
                 module: Optional[ModuleType] = ...) -> None: ...
    def __enter__(self) -> Optional[List[_Record]]: ...
    def __exit__(self, exc_type: Optional[Type[BaseException]],
                 exc_val: Optional[BaseException],
                 exc_tb: Optional[TracebackType]) -> bool: ...



# Stubs for email.errors (Python 3.4)

class MessageError(Exception): ...
class MessageParseError(MessageError): ...
class HeaderParseError(MessageParseError): ...
class BoundaryError(MessageParseError): ...
class MultipartConversionError(MessageError, TypeError): ...

class MessageDefect(ValueError): ...
class NoBoundaryInMultipartDefect(MessageDefect): ...
class StartBoundaryNotFoundDefect(MessageDefect): ...
class FirstHeaderLineIsContinuationDefect(MessageDefect): ...
class MisplacedEnvelopeHeaderDefect(MessageDefect): ...
class MalformedHeaderDefect(MessageDefect): ...
class MultipartInvariantViolationDefect(MessageDefect): ...
class InvalidBase64PaddingDefect(MessageDefect): ...
class InvalidBase64CharactersDefect(MessageDefect): ...
class CloseBoundaryNotFoundDefect(MessageDefect): ...
class MissingHeaderBodySeparatorDefect(MessageDefect): ...



from google.protobuf.internal.containers import (
    RepeatedScalarFieldContainer,
)
from google.protobuf.internal import well_known_types

from google.protobuf.message import (
    Message,
)
from typing import (
    Iterable,
    Optional,
    Text,
)


class FieldMask(Message, well_known_types.FieldMask):
    paths: RepeatedScalarFieldContainer[Text]

    def __init__(self,
                 paths: Optional[Iterable[Text]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> FieldMask: ...



from google.protobuf.descriptor import Descriptor
from google.protobuf.internal.message_listener import MessageListener
from google.protobuf.message import Message
from typing import (
    Sequence, TypeVar, Generic, Any, Iterator, Iterable,
    Union, Optional, Callable, overload, List
)

_T = TypeVar('_T')
class BaseContainer(Sequence[_T]):
    def __init__(self, message_listener: MessageListener) -> None: ...
    def __len__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def sort(self, *, key: Optional[Callable[[_T], Any]] = ..., reverse: bool = ...) -> None: ...
    @overload
    def __getitem__(self, key: int) -> _T: ...
    @overload
    def __getitem__(self, key: slice) -> List[_T]: ...

class RepeatedScalarFieldContainer(BaseContainer[_T]):
    def __init__(self, message_listener: MessageListener, message_descriptor: Descriptor) -> None: ...
    def append(self, value: _T) -> None: ...
    def insert(self, key: int, value: _T) -> None: ...
    def extend(self, elem_seq: Optional[Iterable[_T]]) -> None: ...
    def MergeFrom(self, other: RepeatedScalarFieldContainer[_T]) -> None: ...
    def remove(self, elem: _T) -> None: ...
    def pop(self, key: int = ...) -> _T: ...
    @overload
    def __setitem__(self, key: int, value: _T) -> None: ...
    @overload
    def __setitem__(self, key: slice, value: Iterable[_T]) -> None: ...
    def __getslice__(self, start: int, stop: int) -> List[_T]: ...
    def __setslice__(self, start: int, stop: int, values: Iterable[_T]) -> None: ...
    def __delitem__(self, key: Union[int, slice]) -> None: ...
    def __delslice__(self, start: int, stop: int) -> None: ...

class RepeatedCompositeFieldContainer(BaseContainer[_T]):
    def __init__(self, message_listener: MessageListener, type_checker: Any) -> None: ...
    def add(self, **kwargs: Any) -> _T: ...
    def extend(self, elem_seq: Iterable[_T]) -> None: ...
    def MergeFrom(self, other: RepeatedCompositeFieldContainer[_T]) -> None: ...
    def remove(self, elem: _T) -> None: ...
    def pop(self, key: int = ...) -> _T: ...
    def __getslice__(self, start: int, stop: int) -> List[_T]: ...
    def __delitem__(self, key: Union[int, slice]) -> None: ...
    def __delslice__(self, start: int, stop: int) -> None: ...

# Classes not yet typed
class Mapping(Any): ...
class MutableMapping(Mapping): ...
class ScalarMap(MutableMapping): ...
class MessageMap(MutableMapping): ...



from typing import Any, Union, Text

from Crypto.PublicKey.RSA import _RSAobj

class PKCS115_Cipher:
    def __init__(self, key: _RSAobj) -> None: ...
    def can_encrypt(self) -> bool: ...
    def can_decrypt(self) -> bool: ...
    rf: Any
    def encrypt(self, message: Union[bytes, Text]) -> bytes: ...
    def decrypt(self, ct: bytes, sentinel: Any) -> bytes: ...

def new(key: _RSAobj) -> PKCS115_Cipher: ...



from typing import Any

NO: Any
YES: Any
TRUE: Any
FALSE: Any
ON: Any
OFF: Any
N: Any
S: Any
W: Any
E: Any
NW: Any
SW: Any
NE: Any
SE: Any
NS: Any
EW: Any
NSEW: Any
CENTER: Any
NONE: Any
X: Any
Y: Any
BOTH: Any
LEFT: Any
TOP: Any
RIGHT: Any
BOTTOM: Any
RAISED: Any
SUNKEN: Any
FLAT: Any
RIDGE: Any
GROOVE: Any
SOLID: Any
HORIZONTAL: Any
VERTICAL: Any
NUMERIC: Any
CHAR: Any
WORD: Any
BASELINE: Any
INSIDE: Any
OUTSIDE: Any
SEL: Any
SEL_FIRST: Any
SEL_LAST: Any
END: Any
INSERT: Any
CURRENT: Any
ANCHOR: Any
ALL: Any
NORMAL: Any
DISABLED: Any
ACTIVE: Any
HIDDEN: Any
CASCADE: Any
CHECKBUTTON: Any
COMMAND: Any
RADIOBUTTON: Any
SEPARATOR: Any
SINGLE: Any
BROWSE: Any
MULTIPLE: Any
EXTENDED: Any
DOTBOX: Any
UNDERLINE: Any
PIESLICE: Any
CHORD: Any
ARC: Any
FIRST: Any
LAST: Any
BUTT: Any
PROJECTING: Any
ROUND: Any
BEVEL: Any
MITER: Any
MOVETO: Any
SCROLL: Any
UNITS: Any
PAGES: Any



import socket
import json
import time
from http.server import BaseHTTPRequestHandler, HTTPServer
import requests

import rp

DEFAULT_SERVER_PORT = 25893

class Evaluation:
    def __init__(self,code:str,scope:dict=None):
        if scope is None:
            scope=globals()
        self.scope=scope

        self.code=code
        try:
            self.value=eval(self.code,self.scope,self.scope)
        except KeyboardInterrupt:
            raise
        except BaseException as error:
            self.error=error
            
    @property
    def failed(self):
        return hasattr(self,'error')
    @property
    def succeeded(self):
        return not self.failed
    
    def encode(self):
        return rp.object_to_bytes(self)
    
    def __repr__(self):
        return '<Evaluation: fail=%s>'%self.failed

def _HandlerMaker(scope:dict=None):
    class _Handler(BaseHTTPRequestHandler):
        def do_GET(self):
            self.send_response(200)
            self.send_header("Content-type", "application/octet-stream")
            self.end_headers()
            body=self.get_request_body()
            data=json.loads(body)
            code=data['code']
            print("CODE: "+code)
            evaluation=Evaluation(code,scope)
            response=evaluation.encode()
            self.wfile.write(response)
            
        def get_request_body(self):
            length=int(self.headers.get('Content-Length'))
            body = self.rfile.read(length)
            return body

    return _Handler

def run_server(server_port:int=None,scope:dict=None):
    if server_port is None:
        server_port = DEFAULT_SERVER_PORT

    host_name = "0.0.0.0"
    webServer = HTTPServer((host_name, server_port), _HandlerMaker(scope))
    print("Server started at http://%s:%s" % (rp.get_my_local_ip_address(), server_port))

    try:
        webServer.serve_forever()
    finally:
        webServer.server_close()

    print("Server stopped.")

class Client:
    def __init__(self,server_name:str,server_port:int=None):
        #server_name is like "127.0.1.33" or like "glass.local"
        self.server_name=server_name
        self.server_port = DEFAULT_SERVER_PORT if server_port is None else server_port
        self.server_url='http://%s:%i'%(self.server_name,self.server_port)

    def evaluate(self,code:str):
        response=requests.request('GET',self.server_url,json={'code':code})
        result=rp.bytes_to_object(response.content)
        # assert isinstance(result,Evaluation),'Bad response...make sure the server and client are running on the same version of python with the same library versions'
        return result

if __name__=='__main__':
    if rp.input_yes_no("Is this the server or the client? Yes means this is the server."):
        print("Running the server.")
        run_server()
    else:
        print("Running the client.")
        client=Client(input("Enter the server's IP: "))
        while True:
            print(client.evaluate(input(" >>> ")))



#NEEDS PROMPT TOOLKIT 3
#TAG Prompt Toolkit Imports
from prompt_toolkit import *
from prompt_toolkit.layout import *
from prompt_toolkit.layout.processors import *
from prompt_toolkit.layout.containers import *
from prompt_toolkit.layout.margins import *
from prompt_toolkit.layout.menus import *
from prompt_toolkit.layout.screen import *
from prompt_toolkit.layout.utils import *
from prompt_toolkit.key_binding import *
from prompt_toolkit.buffer import *
from prompt_toolkit.formatted_text import *
from prompt_toolkit.formatted_text import *
from prompt_toolkit.input import *
from prompt_toolkit.utils import *
from prompt_toolkit.application import *
from prompt_toolkit.cache import *
from prompt_toolkit.document import *
from prompt_toolkit.filters import *
from prompt_toolkit.search import *
from prompt_toolkit.auto_suggest import *
from prompt_toolkit.clipboard import *
from prompt_toolkit.completion import *
from prompt_toolkit.contrib import *
from prompt_toolkit.data_structures import *
from prompt_toolkit.enums import *
from prompt_toolkit.eventloop import *
from prompt_toolkit.history import *
from prompt_toolkit.output import *
from prompt_toolkit.patch_stdout import *
from prompt_toolkit.renderer import *
from prompt_toolkit.selection import *
from prompt_toolkit.shortcuts import *
from prompt_toolkit.styles import *
from prompt_toolkit.token import *
from prompt_toolkit.validation import *
from prompt_toolkit.widgets import *
from prompt_toolkit.key_binding.bindings.focus import *
from prompt_toolkit.key_binding.bindings import *
from prompt_toolkit.key_binding.key_bindings import *
from prompt_toolkit.key_binding.bindings.focus import *
from prompt_toolkit.key_binding.bindings import *
from prompt_toolkit.key_binding.key_bindings import *
from functools import partial
from typing import Callable, Generic, List, Optional, Sequence, Tuple, TypeVar, Union

from prompt_toolkit.application.current import get_app
from prompt_toolkit.auto_suggest import AutoSuggest, DynamicAutoSuggest
from prompt_toolkit.buffer import Buffer, BufferAcceptHandler
from prompt_toolkit.completion import Completer, DynamicCompleter
from prompt_toolkit.document import Document
from prompt_toolkit.filters import (
    Condition,
    FilterOrBool,
    has_focus,
    is_done,
    is_true,
    to_filter,
)
from prompt_toolkit.formatted_text import (
    AnyFormattedText,
    StyleAndTextTuples,
    Template,
    to_formatted_text,
)
from prompt_toolkit.formatted_text.utils import fragment_list_to_text
from prompt_toolkit.history import History
from prompt_toolkit.key_binding.key_bindings import KeyBindings
from prompt_toolkit.key_binding.key_processor import KeyPressEvent
from prompt_toolkit.keys import Keys
from prompt_toolkit.layout.containers import (
    AnyContainer,
    ConditionalContainer,
    Container,
    DynamicContainer,
    Float,
    FloatContainer,
    HSplit,
    VSplit,
    Window,
    WindowAlign,
)
from prompt_toolkit.layout.controls import (
    BufferControl,
    FormattedTextControl,
    GetLinePrefixCallable,
)
from prompt_toolkit.layout.dimension import AnyDimension
from prompt_toolkit.layout.dimension import Dimension as D
from prompt_toolkit.layout.dimension import to_dimension
from prompt_toolkit.layout.margins import NumberedMargin, ScrollbarMargin
from prompt_toolkit.layout.processors import (
    AppendAutoSuggestion,
    BeforeInput,
    ConditionalProcessor,
    PasswordProcessor,
    Processor,
)
from prompt_toolkit.lexers import DynamicLexer, Lexer
from prompt_toolkit.mouse_events import MouseEvent, MouseEventType
from prompt_toolkit.utils import get_cwidth
E = KeyPressEvent
_T = TypeVar("_T")











def fuzzy_string_match(string,target,*,case_sensitive=True):
    # >>> fuzzy_string_match('apha','alpha')
    #ans = True
    # >>> fuzzy_string_match('alpha','alpha')
    #ans = True
    # >>> fuzzy_string_match('aa','alpha')
    #ans = True
    # >>> fuzzy_string_match('aqa','alpha')
    #ans = False
    # >>> fuzzy_string_match('e','alpha')
    #ans = False
    # >>> fuzzy_string_match('h','alpha')
    #ans = False
    assert isinstance(string,str)
    assert isinstance(target,str)
    if not case_sensitive:
        string=string.lower()
        target=target.lower()
    import re
    pattern='.*'.join(re.escape(char) for char in string)
    pattern='.*'+pattern+'.*'
    return bool(re.fullmatch(pattern,target))



from rp import *
class FileNavigator():
    #A widget

    def __pt_container__(self):
        #The existence of this function lets us treat this class like a container
        return self.window

    def __init__(self, path='.'):
        

        # self.top_text=Label('',style='bold bg:teal')

        top_text_kb=KeyBindings()
        @top_text_kb.add('enter')
        def _(event):
            self.location=get_relative_path(self.location)
            event.app.layout.focus_next()
        self.top_text=TextArea('',style='bold bg:teal',height=Dimension(max=1),focusable=False)
        self.top_text.control.key_bindings=top_text_kb

        self.original_location=path
        self._set_location(path)

        self._selected_index=0

        # Key bindings.
        kb = KeyBindings()
        kb.add("up"   )(lambda event:self._shift_selected_index(-1))
        kb.add("down" )(lambda event:self._shift_selected_index(+1))
        kb.add("enter")(lambda event:self._enter_handler())
        @kb.add('backspace')
        def _(event):
            # self._set_location(get_parent_directory(self.location))
            self.location=(self.location[:-1])
        @kb.add('escape','backspace')
        def _(event):
            self._set_location(get_parent_directory(self.location))
            # self.location=(self.location[:-1])

        for key in set('qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM./1234567890-_\\|!@#$%^&*()_~`?/'):
            def addkey(key):
                @kb.add(key)
                def _(event):
                    self.location=(self.location+key)
            addkey(key)

        # This container's control
        self.control = FormattedTextControl(self._get_text_fragments,
                                            key_bindings=kb,
                                            focusable   =True)

        # This container's window
        self.navigator= Window(content         =self.control,
                             right_margins     =[ScrollbarMargin(display_arrows=True),],
                             dont_extend_height=True,
                             # width=Dimension(min=15,max=15),
                             )

        self.window=HSplit([self.top_text,self.navigator])

    @property
    def location(self):
        return self.top_text.text
    @location.setter
    def location(self,value):
        self.top_text.text=value

    def _set_location(self,new_location):
        self.location=get_relative_path(new_location,self.original_location)#Simplify the path Turn ./a/../a  to ./a
        self.location=path_join(self.location,'')#Turn asoi/asoid/ada into asoi/asoid/ada/
        self._selected_index=0

    def _enter_handler(self):
        values=self.values.copy()
        if not is_a_folder(self.location):#If we wrote a partial directory fix it when we hit enter
            self.location=get_parent_directory(self.location)
        if values:
            value=values[self._selected_index]
            # print(value)
            new_location=path_join(self.location,value)
            if is_a_folder(new_location):
                self._set_location(new_location)

    def _shift_selected_index(self,amount):
        self._selected_index+=amount
        self._selected_index%=len(self.values)

    def _get_values(self):
        try:
            location=self.location
            if not directory_exists(location):
                name=get_path_name(location)
                location=get_parent_directory(location)
                filter=lambda value:fuzzy_string_match(name,value,case_sensitive=False)
            else:
                filter=lambda value:True

            if not directory_exists(location):
                return ['ERROR: Directory does not exist']

            values=get_all_paths(location            ,
                                  include_folders=True,
                                  include_files  =True,
                                  relative       =True,
                                  sort_by        ='name')

            values = sorted(values,key=lambda path:not is_a_folder(path_join(location,path)))#Put all folders first

            values.insert(0,'..')

            values = [value for value in values if filter(value)]
    
        except Exception as E:
            values=['ERROR: '+str(E)]
            values.insert(0,'..')
        return values

    def _get_text_fragments(self):

        self.values=self._get_values()

        if not self.values:return [] #To prevent an error where we pop from the last element

        result=[]

        location=self.location
        if not is_a_folder(location):
            location=get_parent_directory(location)

        for i, value in enumerate(self.values):
            selected = i == self._selected_index

            style = "yellow" if is_a_folder(path_join(location,value)) else 'green'
            if value.startswith('ERROR: '):
                style='red'
            if selected:
                style += " [SetCursorPosition] bg:darkblue"  

            result.extend(to_formatted_text(value, style=style))
            result.append(("", "\n"))

        result.pop()  # Remove last newline.
        return result

def filenavtest():
    with patch_stdout():
        fn1=FileNavigator()
        fn2=FileNavigator()
        fn3=FileNavigator()
        ans=VSplit([fn1,fn2,fn3],padding_char=' ',padding=0)
        ans=fn1
        ans=Layout(ans)
        def quittableapp(x):
            ans=Application(x)
            kb=KeyBindings()
            @kb.add('escape','q')
            def _(event):
                event.app.exit()
            kb.add('tab')(focus_next)
            kb.add('s-tab')(focus_previous)
            ans.key_bindings=kb
            return ans
        quittableapp(ans,).run ()
filenavtest()






from typing import Any, List, Optional, Tuple, Type
from types import TracebackType
import unittest.case


_SysExcInfoType = Tuple[Optional[Type[BaseException]],
                        Optional[BaseException],
                        Optional[TracebackType]]


class TestResult:
    errors: List[Tuple[unittest.case.TestCase, str]]
    failures: List[Tuple[unittest.case.TestCase, str]]
    skipped: List[Tuple[unittest.case.TestCase, str]]
    expectedFailures: List[Tuple[unittest.case.TestCase, str]]
    unexpectedSuccesses: List[unittest.case.TestCase]
    shouldStop: bool
    testsRun: int
    buffer: bool
    failfast: bool
    tb_locals: bool
    def wasSuccessful(self) -> bool: ...
    def stop(self) -> None: ...
    def startTest(self, test: unittest.case.TestCase) -> None: ...
    def stopTest(self, test: unittest.case.TestCase) -> None: ...
    def startTestRun(self) -> None: ...
    def stopTestRun(self) -> None: ...
    def addError(self, test: unittest.case.TestCase, err: _SysExcInfoType) -> None: ...
    def addFailure(self, test: unittest.case.TestCase, err: _SysExcInfoType) -> None: ...
    def addSuccess(self, test: unittest.case.TestCase) -> None: ...
    def addSkip(self, test: unittest.case.TestCase, reason: str) -> None: ...
    def addExpectedFailure(self, test: unittest.case.TestCase,
                           err: _SysExcInfoType) -> None: ...
    def addUnexpectedSuccess(self, test: unittest.case.TestCase) -> None: ...
    def addSubTest(self, test: unittest.case.TestCase, subtest: unittest.case.TestCase,
                   outcome: Optional[_SysExcInfoType]) -> None: ...
